package com.gic.store.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.store.constant.StoreGroupConstant;
import com.gic.store.dto.StoreStrategyDTO;
import com.gic.store.service.StoreStrategyApiService;
import com.gic.store.utils.ErrorCode;
import com.gic.store.web.qo.PageQO;
import com.gic.store.web.qo.storestrategy.StoreStrategyQO;
import com.gic.store.web.vo.storestrategy.StoreGroupStrateVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author guojx
 * @date 2019/7/4 1:56 PM
 */
@RestController
@RequestMapping("/store-strategy")
public class StoreStrategyController {

    private static final Logger logger = LoggerFactory.getLogger(StoreStrategyController.class);

    @Autowired
    private StoreStrategyApiService storeStrategyApiService;

    @RequestMapping("/save-store-group-strategy")
    public RestResponse saveStoreGroupStrategy(@Validated({StoreStrategyQO.SaveValidView.class, StoreStrategyQO.StoreStrategyTypeValidView.class})
                                                           StoreStrategyQO storeStrategyQO) {
        //分组策略类型
        storeStrategyQO.setStrategyType(StoreGroupConstant.STORE_GROUP_STRATEGY_TYPE);
        ServiceResponse result = storeStrategyApiService.saveStoreStrategy(transferQoToDTO(storeStrategyQO));
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }


    @RequestMapping("/edit-store-group-strategy")
    public RestResponse editStoreGroupStrategy(@Validated({StoreStrategyQO.SaveValidView.class, StoreStrategyQO.RemoveValidView.class})
                                                           StoreStrategyQO storeStrategyQO) {
        ServiceResponse result = storeStrategyApiService.modifyStoreStrategy(transferQoToDTO(storeStrategyQO));
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }

    @RequestMapping("/get-store-group-strategy")
    public RestResponse getStoreGroupStrategy(@Validated({StoreStrategyQO.RemoveValidView.class}) StoreStrategyQO storeStrategyQO) {
        ServiceResponse result = storeStrategyApiService.getStoreStrategyById(storeStrategyQO.getStrategyId());
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }

    @RequestMapping("/delete-store-group-strategy")
    public RestResponse deleteStoreGroupStrategy(@Validated({StoreStrategyQO.RemoveValidView.class}) StoreStrategyQO storeStrategyQO) {
        ServiceResponse result = storeStrategyApiService.deleteStoreStrategy(storeStrategyQO.getStrategyId());
        if (!result.isSuccess()) {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
        return RestResponse.success(result.getResult());
    }


    @RequestMapping("/swap-store-group-strategy")
    public RestResponse swapStoreGroupStrategy(@Validated({StoreStrategyQO.SwapValidView.class, StoreStrategyQO.RemoveValidView.class}) StoreStrategyQO storeStrategyQO) {
        int upOrDown = storeStrategyQO.getUpOrDown();
        ServiceResponse result;
        switch (upOrDown) {
            case StoreGroupConstant.SORT_UP:
                result = storeStrategyApiService.upOrDown(storeStrategyQO.getStrategyId(), true);
                break;
            case StoreGroupConstant.SORT_DOWN:
                result = storeStrategyApiService.upOrDown(storeStrategyQO.getStrategyId(), false);
                break;
            case StoreGroupConstant.SORT_BOTTOM:
                result = storeStrategyApiService.sortBottom(storeStrategyQO.getStrategyId());
                break;
            case StoreGroupConstant.SORT_TOPPING:
                result = storeStrategyApiService.sortTopping(storeStrategyQO.getStrategyId());
                break;
            default:
                result = null;
        }
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/set-store-group-strategy-sort-value")
    public RestResponse setStoreGroupStrategySortValue(@Validated({StoreStrategyQO.SetSortValueValidView.class, StoreStrategyQO.RemoveValidView.class}) StoreStrategyQO storeStrategyQO) {
        ServiceResponse result = storeStrategyApiService.setStoreStrategySort(storeStrategyQO.getStrategyId(), storeStrategyQO.getSortValue());
        if (result.isSuccess()) {
            return RestResponse.success(result.getResult());
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/list-store-group-strategy")
    public RestResponse listStoreGroupStrategy(String search, PageQO pageQO) {
        ServiceResponse<Page<StoreStrategyDTO>> result = storeStrategyApiService.listStoreField(StoreGroupConstant.TEST_ENTERPRISE_ID
                , search, StoreGroupConstant.STORE_GROUP_STRATEGY_TYPE, pageQO.getCurrentPage(), pageQO.getPageSize());
        if (result.isSuccess()) {
            Page<StoreStrategyDTO> resultPage = result.getResult();
            Page<StoreGroupStrateVO> voPage = EntityUtil.changeEntityNew(Page.class, resultPage);
            voPage.setResult(EntityUtil.changeEntityListNew(StoreGroupStrateVO.class, resultPage.getResult()));
            return RestResponse.success(voPage);
        } else {
            return RestResponse.failure(result.getCode(), result.getMessage());
        }
    }

    @RequestMapping("/update-group-strategy-weights")
    public RestResponse updateWeight(Integer weight) {
        if (weight == null) {
            RestResponse.failure(ErrorCode.ERR_5.getCode(), "参数错误");
        }
        ServiceResponse<Integer> result = storeStrategyApiService.setStoreGroupStrategyWeight(StoreGroupConstant.TEST_ENTERPRISE_ID, weight);
        return RestResponse.success(result.getResult());
    }

    @RequestMapping("/get-group-strategy-weights")
    public RestResponse getWeight() {
        ServiceResponse<Integer> result = storeStrategyApiService.getStoreGroupStrategyWeight(StoreGroupConstant.TEST_ENTERPRISE_ID);
        return RestResponse.success(result.getResult());
    }


    private StoreStrategyDTO transferQoToDTO(StoreStrategyQO storeStrategyQO) {
        StoreStrategyDTO storeStrategyDTO = EntityUtil.changeEntityNew(StoreStrategyDTO.class, storeStrategyQO);
        storeStrategyDTO.setEnterpriseId(StoreGroupConstant.TEST_ENTERPRISE_ID);
        return storeStrategyDTO;
    }
}
